@@ -17,12 +17,10 @@
  */
 package org.apache.tuweni.crypto;
 
-import static com.google.common.base.Preconditions.checkArgument;
-import static com.google.common.base.Preconditions.checkNotNull;
-import static com.google.common.base.Preconditions.checkState;
 import static java.nio.file.StandardOpenOption.READ;
 import static org.apache.tuweni.crypto.Hash.keccak256;
 import static org.apache.tuweni.crypto.SECP256K1.Parameters.CURVE;
+import static org.apache.tuweni.crypto.SECP256K1.Parameters.PARAMETER_SPEC;
 import static org.apache.tuweni.io.file.Files.atomicReplace;
 
 import org.apache.tuweni.bytes.Bytes;
@@ -37,16 +35,17 @@ import java.nio.CharBuffer;
 import java.nio.channels.FileChannel;
 import java.nio.file.Path;
 import java.security.InvalidAlgorithmParameterException;
+import java.security.KeyFactory;
 import java.security.KeyPairGenerator;
 import java.security.NoSuchAlgorithmException;
 import java.security.NoSuchProviderException;
 import java.security.SecureRandom;
 import java.security.spec.ECGenParameterSpec;
 import java.util.Arrays;
-import javax.annotation.Nullable;
+import java.util.Objects;
+import javax.crypto.Cipher;
 import javax.security.auth.Destroyable;
 
-import com.google.common.base.Objects;
 import org.bouncycastle.asn1.sec.SECNamedCurves;
 import org.bouncycastle.asn1.x9.X9ECParameters;
 import org.bouncycastle.asn1.x9.X9IntegerConverter;
@@ -59,10 +58,17 @@ import org.bouncycastle.crypto.signers.ECDSASigner;
 import org.bouncycastle.crypto.signers.HMacDSAKCalculator;
 import org.bouncycastle.jcajce.provider.asymmetric.ec.BCECPrivateKey;
 import org.bouncycastle.jcajce.provider.asymmetric.ec.BCECPublicKey;
+import org.bouncycastle.jcajce.provider.asymmetric.util.EC5Util;
+import org.bouncycastle.jcajce.provider.asymmetric.util.ECUtil;
+import org.bouncycastle.jce.provider.BouncyCastleProvider;
+import org.bouncycastle.jce.spec.ECParameterSpec;
+import org.bouncycastle.jce.spec.ECPrivateKeySpec;
+import org.bouncycastle.jce.spec.ECPublicKeySpec;
 import org.bouncycastle.math.ec.ECAlgorithms;
 import org.bouncycastle.math.ec.ECPoint;
 import org.bouncycastle.math.ec.FixedPointCombMultiplier;
 import org.bouncycastle.math.ec.custom.sec.SecP256K1Curve;
+import org.jetbrains.annotations.Nullable;
 
 /*
  * Adapted from the BitcoinJ ECKey (Apache 2 License) implementation:
@@ -91,6 +97,7 @@ public final class SECP256K1 {
   // Lazily initialize parameters by using java initialization on demand
   public static final class Parameters {
     public static final ECDomainParameters CURVE;
+    static final ECParameterSpec PARAMETER_SPEC;
     static final BigInteger CURVE_ORDER;
     static final BigInteger HALF_CURVE_ORDER;
     static final KeyPairGenerator KEY_PAIR_GENERATOR;
@@ -105,6 +112,11 @@ public final class SECP256K1 {
       }
       X9ECParameters params = SECNamedCurves.getByName(CURVE_NAME);
       CURVE = new ECDomainParameters(params.getCurve(), params.getG(), params.getN(), params.getH());
+      PARAMETER_SPEC = new ECParameterSpec(
+              params.getCurve(),
+              CURVE.getG(),
+              CURVE.getN(),
+              CURVE.getH());
       CURVE_ORDER = CURVE.getN();
       HALF_CURVE_ORDER = CURVE_ORDER.shiftRight(1);
       if (CURVE_ORDER.compareTo(SecP256K1Curve.q) >= 0) {
@@ -211,6 +223,42 @@ public final class SECP256K1 {
     return new BigInteger(1, Arrays.copyOfRange(qBytes, 1, qBytes.length));
   }
 
+  /**
+   * Encrypts bytes using a public key.
+   * @param publicKey the public key for encryption
+   * @param payload the payload to encrypt
+   * @return the encrypted data
+   */
+  public static Bytes encrypt(SECP256K1.PublicKey publicKey, Bytes payload) {
+    try {
+      ECPoint ecPoint = publicKey.asEcPoint();
+      ECPublicKeySpec keySpec = new ECPublicKeySpec(ecPoint, PARAMETER_SPEC);
+      KeyFactory keyFactory = KeyFactory.getInstance("EC");
+      java.security.PublicKey bcKey = keyFactory.generatePublic(keySpec);
+
+      Cipher iesCipher = Cipher.getInstance("ECIES", "BC");
+      iesCipher.init(Cipher.ENCRYPT_MODE, bcKey);
+      byte[] output = iesCipher.doFinal(payload.toArrayUnsafe());
+      return Bytes.wrap(output);
+    } catch(Exception e) {
+      throw new EncryptionException(e);
+    }
+  }
+
+  public static Bytes decrypt(SECP256K1.SecretKey secretKey, Bytes encrypted) {
+    try {
+      ECPrivateKeySpec keySpec = new ECPrivateKeySpec(secretKey.bytes().toUnsignedBigInteger(), PARAMETER_SPEC);
+      KeyFactory keyFactory = KeyFactory.getInstance("EC");
+      java.security.PrivateKey bcKey = keyFactory.generatePrivate(keySpec);
+      Cipher iesCipher = Cipher.getInstance("ECIES", "BC");
+      iesCipher.init(Cipher.DECRYPT_MODE, bcKey);
+      byte[] output = iesCipher.doFinal(encrypted.toArrayUnsafe());
+      return Bytes.wrap(output);
+    } catch (Exception e) {
+      throw new DecryptionException(e);
+    }
+  }
+
   /**
    * Generates an ECDSA signature.
    *
@@ -366,8 +414,12 @@ public final class SECP256K1 {
    * @return shared secret as 32 bytes
    */
   public static Bytes32 calculateKeyAgreement(SecretKey privKey, PublicKey theirPubKey) {
-    checkArgument(privKey != null, "missing private key");
-    checkArgument(theirPubKey != null, "missing remote public key");
+    if (privKey == null) {
+      throw new NullPointerException("missing private key");
+    }
+    if (theirPubKey == null) {
+      throw new NullPointerException("missing remote public key");
+    }
 
     ECPrivateKeyParameters privKeyP =
         new ECPrivateKeyParameters(privKey.bytes().toUnsignedBigInteger(), Parameters.CURVE);
@@ -413,7 +465,9 @@ public final class SECP256K1 {
      * @throws IllegalArgumentException If the integer would overflow 32 bytes.
      */
     public static SecretKey fromInteger(BigInteger key) {
-      checkNotNull(key);
+      if (key == null) {
+        throw new NullPointerException("key cannot be null");
+      }
       byte[] bytes = key.toByteArray();
       int offset = 0;
       while (bytes[offset] == 0) {
@@ -474,7 +528,9 @@ public final class SECP256K1 {
     }
 
     private SecretKey(Bytes32 bytes) {
-      checkNotNull(bytes);
+      if (bytes == null) {
+        throw new NullPointerException("bytes cannot be null");
+      }
       this.keyBytes = bytes;
     }
 
@@ -485,7 +541,9 @@ public final class SECP256K1 {
      * @throws IOException On a filesystem error.
      */
     public void store(Path file) throws IOException {
-      checkState(keyBytes != null, "SecretKey has been destroyed");
+      if (keyBytes == null) {
+        throw new NullPointerException("SecretKey has been destroyed");
+      }
       // use buffers for all secret key data transfer, so they can be overwritten on completion
       byte[] bytes = new byte[64];
       CharBuffer hexChars = keyBytes.appendHexTo(CharBuffer.allocate(64));
@@ -506,14 +564,18 @@ public final class SECP256K1 {
       if (!(obj instanceof SecretKey)) {
         return false;
       }
-      checkState(keyBytes != null, "SecretKey has been destroyed");
+      if (keyBytes == null) {
+        throw new NullPointerException("SecretKey has been destroyed");
+      }
       SecretKey other = (SecretKey) obj;
       return this.keyBytes.equals(other.keyBytes);
     }
 
     @Override
     public int hashCode() {
-      checkState(keyBytes != null, "SecretKey has been destroyed");
+      if (keyBytes == null) {
+        throw new NullPointerException("SecretKey has been destroyed");
+      }
       return keyBytes.hashCode();
     }
 
@@ -522,7 +584,9 @@ public final class SECP256K1 {
      * @return The bytes of the key.
      */
     public Bytes32 bytes() {
-      checkState(keyBytes != null, "SecretKey has been destroyed");
+      if (keyBytes == null) {
+        throw new NullPointerException("SecretKey has been destroyed");
+      }
       return keyBytes;
     }
 
@@ -531,7 +595,9 @@ public final class SECP256K1 {
      * @return The bytes of the key.
      */
     public byte[] bytesArray() {
-      checkState(keyBytes != null, "SecretKey has been destroyed");
+      if (keyBytes == null) {
+        throw new NullPointerException("SecretKey has been destroyed");
+      }
       return keyBytes.toArrayUnsafe();
     }
   }
@@ -585,7 +651,9 @@ public final class SECP256K1 {
      * @return The associated public key.
      */
     public static PublicKey fromInteger(BigInteger privateKey) {
-      checkNotNull(privateKey);
+      if (privateKey == null) {
+        throw new NullPointerException("privateKey cannot be null");
+      }
       return fromBytes(toBytes64(privateKey.toByteArray()));
     }
 
@@ -659,8 +727,12 @@ public final class SECP256K1 {
     }
 
     private PublicKey(Bytes bytes) {
-      checkNotNull(bytes);
-      checkArgument(bytes.size() == BYTE_LENGTH, "Key must be %s bytes long, got %s", BYTE_LENGTH, bytes.size());
+      if (bytes == null) {
+        throw new NullPointerException("bytes cannot be null");
+      }
+      if (bytes.size() != BYTE_LENGTH) {
+        throw new IllegalArgumentException(String.format("Key must be %s bytes long, got %s", BYTE_LENGTH, bytes.size()));
+      }
       this.keyBytes = bytes;
     }
 
@@ -786,15 +858,19 @@ public final class SECP256K1 {
     }
 
     private KeyPair(SecretKey secretKey, PublicKey publicKey) {
-      checkNotNull(secretKey);
-      checkNotNull(publicKey);
+      if (secretKey == null) {
+        throw new NullPointerException("secretKey cannot be null");
+      }
+      if (publicKey == null) {
+        throw new NullPointerException("publicKey cannot be null");
+      }
       this.secretKey = secretKey;
       this.publicKey = publicKey;
     }
 
     @Override
     public int hashCode() {
-      return Objects.hashCode(secretKey, publicKey);
+      return Objects.hash(secretKey, publicKey);
     }
 
     @Override
@@ -856,8 +932,12 @@ public final class SECP256K1 {
      * @return The signature.
      */
     public static Signature fromBytes(Bytes bytes) {
-      checkNotNull(bytes);
-      checkArgument(bytes.size() == 65, "Signature must be 65 bytes, but got %s instead", bytes.size());
+      if (bytes == null) {
+        throw new NullPointerException("bytes cannot be null");
+      }
+      if (bytes.size() != 65) {
+        throw new IllegalArgumentException(String.format("Signature must be 65 bytes, but got %s instead", bytes.size()));
+      }
       BigInteger r = bytes.slice(0, 32).toUnsignedBigInteger();
       BigInteger s = bytes.slice(32, 32).toUnsignedBigInteger();
       return new Signature(bytes.get(64), r, s);
@@ -877,19 +957,25 @@ public final class SECP256K1 {
     }
 
     Signature(byte v, BigInteger r, BigInteger s) {
-      checkArgument(v == 0 || v == 1, "Invalid v-value, should be 0 or 1, got %s", v);
-      checkNotNull(r);
-      checkNotNull(s);
-      checkArgument(
-          r.compareTo(BigInteger.ONE) >= 0 && r.compareTo(Parameters.CURVE_ORDER) < 0,
-          "Invalid r-value, should be >= 1 and < %s, got %s",
-          Parameters.CURVE_ORDER,
-          r);
-      checkArgument(
-          s.compareTo(BigInteger.ONE) >= 0 && s.compareTo(Parameters.CURVE_ORDER) < 0,
-          "Invalid s-value, should be >= 1 and < %s, got %s",
-          Parameters.CURVE_ORDER,
-          s);
+      if (v != 0 && v != 1) {
+        throw new IllegalArgumentException(String.format("Invalid v-value, should be 0 or 1, got %s", v));
+      }
+      if (r == null) {
+        throw new NullPointerException("r cannot be null");
+      }
+      if (s == null) {
+        throw new NullPointerException("s cannot be null");
+      }
+      if (r.compareTo(BigInteger.ONE) < 0 || r.compareTo(Parameters.CURVE_ORDER) > 0) {
+        throw new IllegalArgumentException(String.format("Invalid r-value, should be >= 1 and < %s, got %s",
+                Parameters.CURVE_ORDER,
+                r));
+      }
+      if (s.compareTo(BigInteger.ONE) < 0 || s.compareTo(Parameters.CURVE_ORDER) > 0) {
+        throw new IllegalArgumentException(String.format("Invalid s-value, should be >= 1 and < %s, got %s",
+                Parameters.CURVE_ORDER,
+                s));
+      }
       this.v = v;
       this.r = r;
       this.s = s;
@@ -956,7 +1042,7 @@ public final class SECP256K1 {
 
     @Override
     public int hashCode() {
-      return Objects.hashCode(r, s, v);
+      return Objects.hash(r, s, v);
     }
 
     @Override
